/* Star Widget */

@keyframes wiggle {
  12.5% { transform: rotate(15deg); }
  37.5% { transform: rotate(-15deg); }
  62.5% { transform: rotate(15deg); }
  87.5% { transform: rotate(-15deg); }
}

star {
  color: alpha(@theme_fg_color, 0.1);
  transition: color 400ms;
}

star:hover {
  color: alpha(@theme_fg_color, 0.4);
  transition: color 250ms;
}

star:checked {
  color: @theme_selected_bg_color;
  transition: color 250ms;

  animation-name: wiggle;
  animation-duration: 400ms;
  animation-timing-function: linear;
}

/* Popovers with Menus */
popover.custom-menu > contents {
  padding: 12px 8px 8px 8px;
}

popover.custom-menu contents label.title {
  font-weight: bold;
  font-size: 0.8333em;
  opacity: 0.5;
  margin-bottom: 6px;
}

popover.custom-menu list > row {
  border-radius: 5px;
}
