tasklistview row:hover {
  background: none;
}

tasklistview row.top-highlight {
  border-top: 2px solid #2ec27e;
}

tasklistview row.bottom-highlight {
  border-bottom: 2px solid #2ec27e;
}

tasklistview list:drop(active) {
    box-shadow: none;
    border: none;
}

/* Add a subtle shadow to color circles */
image.color-circle-icon {
    -gtk-icon-shadow: 0 1px alpha(black, 0.18);
}

/* Color button */
colorbutton {
    border-radius: 6px;
    transition: 100ms;
}

colorbutton:hover {
    filter: brightness(80%);
    transition: 100ms;
}

colorbutton.dark image {
    color: white;
}

colorbutton.light image {
    color: black;
}

