@import url("resource:///org/gnome/todo/themes/Adwaita-omniarea.css");
@import url("resource:///org/gnome/todo/themes/Adwaita-tasklistview.css");
@import url("resource:///org/gnome/todo/themes/Adwaita-taskrow.css");
@import url("resource:///org/gnome/todo/themes/Adwaita-themeselector.css");
@import url("resource:///org/gnome/todo/themes/Adwaita-widgets.css");

.transparent {
    background-color: transparent;
}

row.thumbnail {
    border: solid 1px @borders;
    box-shadow: 0px 0px 3px alpha(@borders, 0.8);
    border-radius: 16px;
}

grid-item.thumbnail {
    padding: 6px;
    font-size: 10px;
}

grid-item image {
    margin: 12px 24px;
    font-size: 10px;
    border: solid 1px @borders;
    border-radius: 4px;
    box-shadow: 0px 0px 4px alpha(@borders, 0.8);
}

grid-item {
    background-color: transparent;
}

grid-item.light {
    color: #000003;
}

grid-item.dark {
    color: #eeeeec;
}

arrow-frame {
    border: solid 1px;
    border-bottom-width: 0px;
    border-top-width: 0px;
    background-color: @theme_bg_color;
    border-color: @borders;
}

arrow-frame:dir(ltr) {
    border-right-width: 0px;
}

arrow-frame:dir(rtl) {
    border-left-width: 0px;
}

label.main-title {
    font-size: 28px;
}
