omniarea entry {
  background-color: mix(@theme_bg_color, @content_view_bg, 0.25);
  color: @theme_fg_color;
}
omniarea:hover entry,
omniarea:active entry {
  background-color: @content_view_bg;
  color: @theme_fg_color;
}

omniarea:backdrop entry {
  background-color: @theme_bg_color;
}
