taskrow {
  border-radius: 5px;
  transition: 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
}

taskrow:hover {
  background-color: alpha(@theme_fg_color, 0.1);
}

taskrow:dir(ltr) { padding-left: 6px;  }
taskrow:dir(rtl) { padding-right: 6px; }

taskrow.active {
  background-color: @theme_bg_color;
  border: solid 1px @borders;
}

/* task title entry */
taskrow text {
  border: solid 1px transparent;
  background: none;
  border-radius: 3px;
}

taskrow text:hover:not(:focus) {
  border: solid 1px @borders;
}

/* line-through complete tasks */
taskrow.complete > * { opacity: 0.5; }
taskrow.complete entry.title { text-decoration-line: line-through; }

/* dnd row */
taskrow box.dnd {
    background: alpha(@theme_fg_color, 0.1);
}

tasklistview { background-color: @theme_base_color; }

/* new task row */
newtaskrow entry { padding: 0 16px 0 14px; }

/* extension list */
list.extension-list row {
    border-bottom: solid 1px @borders;
}

/* circle button */
button.circle {
    border-radius: 20px;
}

