themeselector checkbutton radio {
  -gtk-icon-source: none;
  background: none;
  box-shadow: none;
  padding: 12px;
  min-height: 24px;
  min-width: 24px;
  border-width: 3px;
  border-style: solid;
  border-color: transparent;
}

themeselector checkbutton radio:checked {
  border-color: @theme_selected_bg_color;
  -gtk-icon-source: -gtk-scaled(-gtk-icontheme("object-select-symbolic"));
}

themeselector checkbutton.light radio {
  color: #2e3436;
  background-color: #fff;
}
themeselector checkbutton.dark radio {
  color: #fff;
  background-color: #2e3436;
}
